@@ -20,7 +20,7 @@ def keygen(bit_length):
     return params, pk, sk
 
 def encrypt(params, pk, m):
-    if isinstance(m, np.matrix):
+    if isinstance(m, np.matrix) or isinstance(m, list):
         m = np.array(m)
 
     # scalar
@@ -44,7 +44,7 @@ def encrypt(params, pk, m):
         return None
 
 def decrypt(params, sk, c):
-    if isinstance(c, np.matrix):
+    if isinstance(c, np.matrix) or isinstance(c, list):
         c = np.array(c)
 
     # scalar
@@ -68,9 +68,9 @@ def decrypt(params, sk, c):
         return None
 
 def mult(params, c1, c2):
-    if isinstance(c1, np.matrix):
+    if isinstance(c1, np.matrix) or isinstance(c1, list):
         c1 = np.array(c1)
-    if isinstance(c2, np.matrix):
+    if isinstance(c2, np.matrix) or isinstance(c2, list):
         c2 = np.array(c2)
 
     # scalar x scalar
@@ -128,7 +128,7 @@ def dec(params, sk, c, delta):
     return decode(params, decrypt(params, sk, c), delta)
 
 def dec_add(params, sk, c, delta):
-    if isinstance(c, np.matrix):
+    if isinstance(c, np.matrix) or isinstance(c, list):
         c = np.array(c)
 
     # scalar
@@ -159,7 +159,7 @@ def _decrypt(params, sk, c):
     return (minv(mpow(c[0], sk, params.p), params.p) * c[1]) % params.p
 
 def _mult(params, c1, c2):
-    return [(c1[0] * c2[0]) % params.p, (c1[1] * c2[1]) % params.p]
+    return np.array([(c1[0] * c2[0]) % params.p, (c1[1] * c2[1]) % params.p], dtype=object)
 
 def _encode(params, x, delta, mode):
     if mode == 'nearest':
